@@ -13,6 +13,7 @@ from dvc.exceptions import (
     MergeError,
     RemoteCacheRequiredError,
 )
+from dvc.hash_info import HashInfo
 
 from ..tree.base import BaseTree
 
@@ -113,7 +114,7 @@ class BaseOutput:
         self.stage = stage
         self.repo = stage.repo if stage else None
         self.def_path = path
-        self.info = info
+        self.hash_info = HashInfo.from_dict(info)
         if tree:
             self.tree = tree
         else:
@@ -162,7 +163,7 @@ class BaseOutput:
 
     @property
     def dir_cache(self):
-        return self.cache.get_dir_cache(self.checksum)
+        return self.cache.get_dir_cache(self.hash_info)
 
     @classmethod
     def supported(cls, url):
@@ -170,42 +171,31 @@ class BaseOutput:
 
     @property
     def cache_path(self):
-        return self.cache.tree.hash_to_path_info(self.checksum).url
+        return self.cache.tree.hash_to_path_info(self.hash_info.value).url
 
-    @property
-    def checksum_type(self):
-        return self.tree.PARAM_CHECKSUM
-
-    @property
-    def checksum(self):
-        return self.info.get(self.tree.PARAM_CHECKSUM)
-
-    @checksum.setter
-    def checksum(self, checksum):
-        self.info[self.tree.PARAM_CHECKSUM] = checksum
-
-    def get_checksum(self):
-        return self.tree.get_hash(self.path_info)[1]
+    def get_hash(self):
+        if not self.use_cache:
+            return self.tree.get_hash(self.path_info)
+        return self.cache.get_hash(self.tree, self.path_info)
 
     @property
     def is_dir_checksum(self):
-        return self.tree.is_dir_hash(self.checksum)
+        return self.hash_info.isdir
 
     @property
     def exists(self):
         return self.tree.exists(self.path_info)
 
-    def save_info(self):
-        return self.tree.save_info(self.path_info)
-
     def changed_checksum(self):
-        return self.checksum != self.get_checksum()
+        return self.hash_info != self.get_hash()
 
     def changed_cache(self, filter_info=None):
-        if not self.use_cache or not self.checksum:
+        if not self.use_cache or not self.hash_info:
             return True
 
-        return self.cache.changed_cache(self.checksum, filter_info=filter_info)
+        return self.cache.changed_cache(
+            self.hash_info, filter_info=filter_info
+        )
 
     def workspace_status(self):
         if not self.exists:
@@ -214,13 +204,13 @@ class BaseOutput:
         if self.changed_checksum():
             return {str(self): "modified"}
 
-        if not self.checksum:
+        if not self.hash_info:
             return {str(self): "new"}
 
         return {}
 
     def status(self):
-        if self.checksum and self.use_cache and self.changed_cache():
+        if self.hash_info and self.use_cache and self.changed_cache():
             return {str(self): "not in cache"}
 
         return self.workspace_status()
@@ -275,7 +265,7 @@ class BaseOutput:
             self.verify_metric()
 
         if not self.use_cache:
-            self.info = self.save_info()
+            self.hash_info = self.get_hash()
             if not self.IS_DEPENDENCY:
                 logger.debug(
                     "Output '%s' doesn't use cache. Skipping saving.", self
@@ -288,15 +278,15 @@ class BaseOutput:
             logger.debug("Output '%s' didn't change. Skipping saving.", self)
             return
 
-        self.info = self.save_info()
+        self.hash_info = self.get_hash()
 
     def commit(self):
-        assert self.info
+        assert self.hash_info
         if self.use_cache:
-            self.cache.save(self.path_info, self.cache.tree, self.info)
+            self.cache.save(self.path_info, self.cache.tree, self.hash_info)
 
     def dumpd(self):
-        ret = copy(self.info)
+        ret = copy(self.hash_info.to_dict())
         ret[self.PARAM_PATH] = self.def_path
 
         if self.IS_DEPENDENCY:
@@ -345,7 +335,7 @@ class BaseOutput:
 
         return self.cache.checkout(
             self.path_info,
-            self.info,
+            self.hash_info,
             force=force,
             progress_callback=progress_callback,
             relink=relink,
@@ -379,7 +369,7 @@ class BaseOutput:
             return 0
 
         return self.cache.get_files_number(
-            self.path_info, self.checksum, filter_info
+            self.path_info, self.hash_info, filter_info
         )
 
     def unprotect(self):
@@ -389,9 +379,9 @@ class BaseOutput:
     def get_dir_cache(self, **kwargs):
         if not self.is_dir_checksum:
             raise DvcException("cannot get dir cache for file checksum")
-        if self.cache.changed_cache_file(self.checksum):
+        if self.cache.changed_cache_file(self.hash_info):
             self.repo.cloud.pull(
-                NamedCache.make("local", self.checksum, str(self)),
+                NamedCache.make("local", self.hash_info.value, str(self)),
                 show_checksums=False,
                 **kwargs,
             )
@@ -426,7 +416,7 @@ class BaseOutput:
         except DvcException:
             logger.debug(f"failed to pull cache for '{self}'")
 
-        if self.cache.changed_cache_file(self.checksum):
+        if self.cache.changed_cache_file(self.hash_info):
             msg = (
                 "Missing cache for directory '{}'. "
                 "Cache for files inside will be lost. "
@@ -476,7 +466,7 @@ class BaseOutput:
             cache.external[dep.repo_pair].add(dep.def_path)
             return cache
 
-        if not self.checksum:
+        if not self.hash_info:
             msg = (
                 "Output '{}'({}) is missing version info. "
                 "Cache for it will not be collected. "
@@ -495,13 +485,13 @@ class BaseOutput:
             logger.warning(msg)
             return NamedCache()
 
-        ret = NamedCache.make(self.scheme, self.checksum, str(self))
+        ret = NamedCache.make(self.scheme, self.hash_info.value, str(self))
 
         if not self.is_dir_checksum:
             return ret
 
         ret.add_child_cache(
-            self.checksum, self.collect_used_dir_cache(**kwargs),
+            self.hash_info.value, self.collect_used_dir_cache(**kwargs),
         )
 
         return ret
@@ -543,11 +533,13 @@ class BaseOutput:
 
         if ancestor:
             self._check_can_merge(ancestor)
-            ancestor_info = ancestor.info
+            ancestor_info = ancestor.hash_info
         else:
             ancestor_info = None
 
         self._check_can_merge(self)
         self._check_can_merge(other)
 
-        self.info = self.cache.merge(ancestor_info, self.info, other.info)
+        self.hash_info = self.cache.merge(
+            ancestor_info, self.hash_info, other.hash_info
+        )
